// ##########################################################################
// ####                                                                  ####
// ####                         Prfungsamt-Client                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Attribut-Editor                                                  ####
// ####                                                                  ####
// #### Version 1.00  --  12. Juli 2000                                  ####
// ####                                                                  ####
// #### Copyright (C) 2000  Thomas Dreibholz                             ####
// ####                     Universitt Bonn                             ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "attributeeditor.h"
#include "sqlmessages.h"


#include <qlayout.h>
#include <qpushbutton.h>
#include <qcombobox.h>


#include "attributeeditor.moc"


// ###### Constructor #######################################################
AttributeEditor::AttributeEditor(SQLConnectionInterface* connection,
                                 const ViewEntry*        view,
                                 const QString&          attribute,
                                 const bool              editable,
                                 const QString&          value,
                                 QWidget*                parent,
                                 const char*             name)
   : QWidget(parent,name)
{
   Connection = connection;
   View       = view;
   Attribute  = attribute;


   // ====== Widget initialisieren ==========================================
   LineEdit   = NULL;
   Label      = NULL;
   ComboBox   = NULL;
   Button     = NULL;
   Selector   = NULL;
   SelectList = NULL;

   QHBoxLayout* layout = new QHBoxLayout(this);
   CHECK_PTR(layout);


   // ====== LineEdit fr deb Attributwert ==================================
   if(editable) {
      LineEdit = new QLineEdit(this);
      CHECK_PTR(LineEdit);
      LineEdit->setText(value);
      LineEdit->setMinimumSize(LineEdit->sizeHint());
      layout->addWidget(LineEdit,10);
      QObject::connect(LineEdit,SIGNAL(returnPressed()),this,SLOT(returnPressed()));


      // ====== PushButton fr Werte-Auswahl, falls mglich =================
      for(cardinal j = 0;j < View->Selects;j++) {
         if(View->Select[j].SelectAttributeR1 == attribute) {
            // ====== Auswahl-Button erzeugen ===============================
            if(!View->Select[j].UseComboBox) {
               Button = new QPushButton("Auswahl...",this);
               CHECK_PTR(Button);
               Button->setMinimumSize(Button->sizeHint());
               layout->addWidget(Button,10);
               QObject::connect(Button,SIGNAL(clicked()),this,SLOT(clicked()));
            }
            else {
               ComboBox = new QComboBox(this);
               CHECK_PTR(ComboBox);
               ComboBox->setMinimumSize(ComboBox->sizeHint());
               ComboBox->setMinimumHeight(LineEdit->height());
               layout->addWidget(ComboBox,10);
               QObject::connect(ComboBox,SIGNAL(activated(int)),this,SLOT(activated(int)));
            }

            // ====== Wert-Informationen aus Datenbank lesen ================
            SelectValue          = value;
            SelectView           = NULL;
            SelectInfoAttributes = "";
            for(cardinal i = 0;i < View->Selects;i++) {
               if(View->Select[i].SelectAttributeR1 == Attribute) {
                  SelectAttribute      = View->Select[i].SelectAttributeR2;
                  SelectInfoAttributes = View->Select[i].SelectInfoAttributes;
                  SelectView           = View->Select[i].View;
               }
            }
            if(SelectView != NULL) {
               if((Button) && (value.length() > 0)) {
                  QString result = fetchTuple(SelectView->BaseTableName,
                                              SelectInfoAttributes,
                                              SelectAttribute,value);
                  if(result != value) {
                     Button->setText(result);
                  }
               }
               else if(ComboBox) {
                  SelectList = fetchValueList(SelectView->BaseTableName,
                                              SelectInfoAttributes,
                                              SelectAttribute);
                  if(SelectList != NULL) {
                     integer selected = -1;
                     for(cardinal i = 0;i < SelectList->count();i++) {
                        ComboBox->insertItem(SelectList->at(i)->Value,i);
                        if(SelectList->at(i)->Key == SelectValue) {
                           selected = (integer)i;
                        }
                     }
                     ComboBox->insertItem("",SelectList->count());
                     if(selected == -1) {
                        ComboBox->setCurrentItem(SelectList->count());
                     }
                     else {
                        ComboBox->setCurrentItem(selected);
                     }
                  }
               }
            }

            break;
         }
      }
   }

   // ====== Label fr nicht-editierbaren Wert ==============================
   else {
      Label = new QLabel(value,this);
      CHECK_PTR(Label);
      Label->setMinimumSize(Label->sizeHint());
      layout->addWidget(Label);
   }

}


// ###### Destructor ########################################################
AttributeEditor::~AttributeEditor()
{
   if(Selector) {
      delete Selector;
      Selector = NULL;
   }
   if(SelectList) {
      delete SelectList;
      SelectList = NULL;
   }
}


// ###### Attributwert auslesen #############################################
QString AttributeEditor::getValue() const
{
   if(LineEdit != NULL) {
      return(LineEdit->text());
   }
   else if(Label != NULL) {
      return(Label->text());
   }
   return(NULL);
}


// ###### Attributwert setzen ###############################################
void AttributeEditor::setValue(const QString& value)
{
   if(LineEdit != NULL) {
      LineEdit->setText(value);
   }
   else if(Label != NULL) {
      Label->setText(value);
   }
}


// ###### Attributwert markieren ############################################
void AttributeEditor::mark()
{
   if(LineEdit != NULL) {
      raise();
      LineEdit->setFocus();
      LineEdit->selectAll();
   }
}


// ###### Return gedrckt ###################################################
void AttributeEditor::returnPressed()
{
   emit userUpdate();
}


// ###### Tupel aus Datenbank lesen #########################################
QString AttributeEditor::fetchTuple(const QString& tableName,
                                    const QString& attributes,
                                    const QString& key,
                                    const QString& keyValue)
{
   QString tuple = keyValue;

   // ====== Transaktion beginnen ===========================================
   try {
      Connection->beginTransaction();

      // ====== Daten aus Datenbank holen ===================================
      QString command = "SELECT " + attributes + " FROM " + tableName +
                        " WHERE " + key + " = '" + keyValue + "'";
      Connection->createCursor("fetchTuple_cursor",command);
      Connection->fetchCursor("fetchTuple_cursor","1");

      // ====== Tupel-String erzeugen =======================================
      if(Connection->getTuples() == 1) {
         const cardinal fields = Connection->getFields();
         tuple = "";
         for(cardinal i = 0;i < fields;i++) {
            const QString value = Connection->getValue(0,i);
            tuple += QString(value.stripWhiteSpace() + " ");
         }
      }
/*
      else {
         cerr << "AttributeEditor::fetchTuple("
              << tableName
              << ") - Kein passendes Tupel gefunden!" << endl;
      }
*/
      Connection->deleteCursor("fetchTuple_cursor");
      Connection->commitTransaction();
   }

   // ====== Fehler abfangen ================================================
   catch(SQLException e) {
      Connection->rollbackTransaction();
      sqlWarning(this,e.toString());
   }

   return(tuple);
}


QList<AttributeEditor::ValueListEntry>* AttributeEditor::fetchValueList(
                                           const QString& tableName,
                                           const QString& attributes,
                                           const QString& key)
{
   // ====== Liste erzeugen =================================================
   QList<AttributeEditor::ValueListEntry>* list = new QList<AttributeEditor::ValueListEntry>;
   CHECK_PTR(list);
   list->setAutoDelete(true);

   // ====== Transaktion beginnen ===========================================
   try {
      Connection->beginTransaction();

      // ====== Daten aus Datenbank holen ===================================
      QString command = "SELECT " + key + ", " + attributes + " FROM " + tableName;
      Connection->createCursor("fetchTuple_cursor",command);
      Connection->fetchCursor("fetchTuple_cursor","ALL");

      // ====== Tupel-String erzeugen =======================================
      const cardinal tuples = Connection->getTuples();
      const cardinal fields = Connection->getFields();
      for(cardinal i = 0;i < tuples;i++) {
         ValueListEntry* entry = new ValueListEntry;
         CHECK_PTR(entry);
         const QString key = Connection->getValue(i,0);
         entry->Key = key.stripWhiteSpace();
         for(cardinal j = 1;j < fields;j++) {
            if(j > 1) {
               entry->Value += " ";
            }
            const QString value = Connection->getValue(i,j);
            entry->Value += value.stripWhiteSpace();
         }
         list->append(entry);
      }
      Connection->deleteCursor("fetchTuple_cursor");
      Connection->commitTransaction();
   }

   // ====== Fehler abfangen ================================================
   catch(SQLException e) {
      Connection->rollbackTransaction();
      sqlWarning(this,e.toString());
      delete list;
      list = NULL;
   }

   return(list);
}


// ###### Qt-Slot: Selektion in ComboBox ausgefhrt #########################
void AttributeEditor::activated(int index)
{
   ValueListEntry* entry = SelectList->at(index);
   if(entry != NULL) {
      LineEdit->setText(entry->Key);
   }
}


// ###### Qt-Slot: Selektion starten ########################################
void AttributeEditor::clicked()
{
   if(Selector != NULL) {
      if(Selector->isVisible()) {
         Selector->raise();
         return;
      }
      delete Selector;
      Selector = NULL;
   }

   // ====== Selektion vorbereiten ==========================================
   SelectValue          = (LineEdit != NULL) ? LineEdit->text() : QString("");
   SelectView           = NULL;
   SelectInfoAttributes = "";
   for(cardinal i = 0;i < View->Selects;i++) {
      if(View->Select[i].SelectAttributeR1 == Attribute) {
         SelectAttribute      = View->Select[i].SelectAttributeR2;
         SelectInfoAttributes = View->Select[i].SelectInfoAttributes;
         SelectView           = View->Select[i].View;
      }
   }
   if(SelectView == NULL) {
      cerr << "FEHLER: AttributeEditor::clicked() - Kein Selektionsattribut!" << endl;
      exit(1);
   }

   // ====== Selektion durchfhren ==========================================
   Selector = new TableViewer(Connection,
                              SelectView, TableViewer::TVM_SelectMode,
                              NULL, NULL, SelectAttribute, SelectValue);
   CHECK_PTR(Selector);
   Selector->loadTable();
   Selector->show();
   QObject::connect(Selector,SIGNAL(doneSelection(TableViewer*,const bool,const QString&)),this,SLOT(valueSelected(TableViewer*,const bool,const QString&)));
}


// ###### Qt-Slot: Selektion in TableViewer durchgefhrt ####################
void AttributeEditor::valueSelected(TableViewer* viewer, const bool selected, const QString& selection)
{
   if(Selector != NULL) {
      if(selected == true) {
         QString result = fetchTuple(SelectView->BaseTableName,
                                     SelectInfoAttributes,
                                     SelectAttribute,selection);
         Button->setText(result);
         LineEdit->setText(selection);
      }
      Selector->hide();
   }
}
