// ##########################################################################
// ####                                                                  ####
// ####                         Prfungsamt-Client                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Tabellen-Viewer                                                  ####
// ####                                                                  ####
// #### Version 1.00  --  12. Juli 2000                                  ####
// ####                                                                  ####
// #### Copyright (C) 2000  Thomas Dreibholz                             ####
// ####                     Universitt Bonn                             ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "tableviewer.h"
#include "sqlmessages.h"


#include <qheader.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qhbuttongroup.h>
#include <qlayout.h>


#include "tableviewer.moc"


// ###### Constructor #######################################################
TableViewer::TableViewer(SQLConnectionInterface* connection,
                         const ViewTable*        viewTable,
                         const cardinal          mode,
                         const QString&          joinAttribute,
                         const QString&          joinValue,
                         const QString&          selectAttribute,
                         const QString&          selectValue,
                         QWidget*                parent,
                         const char*             name)
   : QWidget(parent,name)
{
   Connection       = connection;
   Views            = viewTable;
   View             = &Views->View[0];
   Mode             = mode;
   JoinAttribute    = joinAttribute;
   JoinValue        = joinValue;
   SelectAttribute  = selectAttribute;
   SelectValue      = selectValue;
   SkipNextUpdate   = false;
   MinNotFoundIndex = 0;
   Attributes.setAutoDelete(true);
   TupleEditors.setAutoDelete(true);


   // ====== Widget initialisieren ==========================================
   WhatsThis = new QWhatsThis(this);
   CHECK_PTR(WhatsThis);
   WhatsThis->add(this,"Willkommen im Prfungsamt-Client!");
   QGridLayout* topLayout = new QGridLayout(this,4,1,5);
   CHECK_PTR(topLayout);
   topLayout->setRowStretch(0,0);
   topLayout->setRowStretch(1,10);
   topLayout->setRowStretch(2,0);


   // ====== Kontroll-Zeile =================================================
   QWidget* controlGroup = new QWidget(this);
   CHECK_PTR(controlGroup);
   WhatsThis->add(controlGroup,"In dieser Gruppe kann eine Sicht ausgewhlt, "
                               "ein neues Tupel eingefgt und nach einem "
                               "Prfix gesucht werden.");
   QHBoxLayout* controlLayout = new QHBoxLayout(controlGroup,5);
   CHECK_PTR(controlLayout);

   QComboBox* viewCombo = new QComboBox(controlGroup);
   CHECK_PTR(viewCombo);
   QPushButton* newButton = new QPushButton("Neues Tupel...",controlGroup);
   CHECK_PTR(newButton);
   if((!(Mode & TVM_EditMode)) || (Mode & TVM_ReadOnly)) {
      newButton->setEnabled(false);
   }

   FindLabel = new QLabel("Auswahl: undefiniert",controlGroup);
   CHECK_PTR(FindLabel);
   FindLineEdit = new QLineEdit(controlGroup);
   CHECK_PTR(FindLineEdit);
   FindLineEdit->setMaxLength(128);

   WhatsThis->add(viewCombo,"Hier kann eine Sicht ausgewhlt werden.");
   WhatsThis->add(FindLabel,"Hier ist das Attribut angegeben, fr das die Prfix-Suche "
                            "durchgefhrt wird. Es kann durch anklicken eines anderen "
                            "Attributes im Tabellen-Titel gendert werden.");
   WhatsThis->add(FindLineEdit,"Hier kann ein zu suchendes Prfix angegeben werden.");

   for(cardinal i = 0;i < Views->Views;i++) {
      viewCombo->insertItem(Views->View[i].Title,i);
   }


   FindLabel->setMinimumSize(FindLabel->sizeHint());
   FindLineEdit->setMinimumSize(FindLineEdit->sizeHint());
   viewCombo->setMinimumSize(viewCombo->sizeHint());
   newButton->setMinimumSize(newButton->sizeHint());
   viewCombo->setMinimumHeight(newButton->height());
   controlLayout->addWidget(viewCombo);
   controlLayout->addWidget(newButton);
   controlLayout->addWidget(FindLabel);
   controlLayout->addWidget(FindLineEdit);
   controlGroup->setMinimumSize(controlGroup->sizeHint());
   topLayout->addWidget(controlGroup,0,0);


   // ====== QListView ======================================================
   TableView = new QListView(this);
   CHECK_PTR(TableView);
   TableView->setMinimumSize(680,350);
   TableView->setShowSortIndicator(true);
   TableView->setAllColumnsShowFocus(true);
   topLayout->addWidget(TableView,1,0);

   if(Mode & TVM_SelectMode) {
      QHButtonGroup* databaseGroup = new QHButtonGroup(this);
      CHECK_PTR(databaseGroup);
      QPushButton* okButton = new QPushButton("Okay",databaseGroup);
      CHECK_PTR(okButton);
      okButton->setMinimumSize(okButton->sizeHint());
      QPushButton* nullButton = new QPushButton("Auf NULL setzen",databaseGroup);
      CHECK_PTR(nullButton);
      nullButton->setMinimumSize(nullButton->sizeHint());
      QPushButton* cancelButton = new QPushButton("Abbruch",databaseGroup);
      CHECK_PTR(cancelButton);
      cancelButton->setMinimumSize(cancelButton->sizeHint());

      topLayout->addWidget(databaseGroup,2,0);

      QObject::connect(okButton,SIGNAL(clicked()),this,SLOT(okayClicked()));
      QObject::connect(nullButton,SIGNAL(clicked()),this,SLOT(nullClicked()));
      QObject::connect(cancelButton,SIGNAL(clicked()),this,SLOT(cancelClicked()));
   }

   // ====== Status-Zeile ===================================================
   StatusBar = new QLabel(this);
   CHECK_PTR(StatusBar);
   WhatsThis->add(StatusBar,"Dies ist die Status-Zeile.");
   StatusBar->setMinimumSize(StatusBar->sizeHint());
   topLayout->addWidget(StatusBar,3,0);


   // ====== Signale mit Slots verbinden ====================================
   QObject::connect(FindLineEdit,SIGNAL(textChanged(const QString&)),this,SLOT(search(const QString&)));
   QObject::connect(FindLineEdit,SIGNAL(returnPressed()),TableView,SLOT(setFocus()));
   QObject::connect(TableView->header(),SIGNAL(clicked(int)),this,SLOT(setSortAttribute(int)));
   QObject::connect(TableView,SIGNAL(doubleClicked(QListViewItem*)),this,SLOT(selectTuple(QListViewItem*)));
   QObject::connect(TableView,SIGNAL(returnPressed(QListViewItem*)),this,SLOT(selectTuple(QListViewItem*)));
   QObject::connect(viewCombo,SIGNAL(activated(int)),this,SLOT(selectView(int)));
   QObject::connect(newButton,SIGNAL(clicked()),this,SLOT(newClicked()));
   TableView->header()->setClickEnabled(true);
}


// ###### Destructor ########################################################
TableViewer::~TableViewer()
{
   delete WhatsThis;
   WhatsThis = NULL;
}


// ###### Qt-Event: Fenster schlieen #######################################
void TableViewer::closeEvent(QCloseEvent* event)
{
   event->accept();
   emit doneSelection(this,false,NULL);
}


// ###### Qt-Slot: Neu-Button angeklickt ####################################
void TableViewer::newClicked()
{
   // ====== Ist schon Neu-Fenster auf dem Bildschirm? ======================
   const char* NewID = "--NEW--";
   TupleEditor* editor = TupleEditors[NewID];
   if(editor != NULL) {
      editor->raise();
      return;
   }
   if((Attributes.count() <= 0) || (Mode & TVM_ReadOnly)) {
      return;
   }


   // ====== Tupeldaten initialisieren ======================================
   Tuple* tuple = new Tuple;
   CHECK_PTR(tuple);

   tuple->ID         = NewID;
   tuple->TableName  = Views->BaseTableName;
   tuple->View       = View;
   tuple->Attributes = Attributes.count();

   integer keys = 0;
   for(cardinal i = 0;i < tuple->Attributes;i++) {
      tuple->Attribute[i] = *(Attributes.at(i));
      tuple->IsKey[i]     = false;
      if(tuple->Attribute[i] == JoinAttribute) {
         tuple->Value[i] = JoinValue;
      }
      else {
         tuple->Value[i] = "";
      }

      for(cardinal j = 0;j < ViewTable::MaxPrimaryKeyEntries;j++) {
         if((Views->PrimaryKey[j] != NULL) && (*(Attributes.at(i)) == Views->PrimaryKey[j])) {
            tuple->IsKey[i] = true;
            keys++;
            break;
         }
      }
   }
   for(cardinal i = 0;i < ViewTable::MaxPrimaryKeyEntries;i++) {
      if(Views->PrimaryKey[i] != NULL) {
         keys--;
      }
   }
   if(keys != 0) {
      cerr << "FEHLER: TableViewer::newClicked(" << Views->BaseTableName
           << ") - Primrschlssel-Angaben sind fehlerhaft!" << endl;
      exit(1);
   }


   // ====== Editor ffnen ==================================================
   editor = new TupleEditor(Connection,tuple,true,JoinAttribute);
   CHECK_PTR(editor);
   editor->show();
   QObject::connect(editor,SIGNAL(done(TupleEditor*, const bool)),this,SLOT(closeEditor(TupleEditor*, const bool)));
   TupleEditors.insert(tuple->ID,editor);
}


// ###### Qt-Slot: Sicht ausgewhlt #########################################
void TableViewer::selectView(int selection)
{
   if(selection < (int)Views->Views) {
      View = &Views->View[selection];
      loadTable(View->ViewName,View->OrderBy);
   }
}


// ###### Qt-Slot: Okay-Button angeklickt ###################################
void TableViewer::okayClicked()
{
   selectTuple(TableView->currentItem());
}


// ###### Qt-Slot: Null-Button angeklickt ###################################
void TableViewer::nullClicked()
{
   emit doneSelection(this,true,"");
}


// ###### Qt-Slot: Cancel-Button angeklickt #################################
void TableViewer::cancelClicked()
{
   emit doneSelection(this,false,NULL);
}


// ###### Qt-Slot: Tupel ausgewhlt #########################################
void TableViewer::selectTuple(QListViewItem* item)
{

   if(item != NULL) {
      // ====== Selektier-Modus =============================================
      if(Mode & TVM_SelectMode) {
         for(cardinal i = 0;i < Attributes.count();i++) {
            if(*(Attributes.at(i)) == SelectAttribute) {
               emit doneSelection(this,true,item->text(i));
               return;
            }
         }
      }

      // ====== Editier-Modus ===============================================
      else if((Mode & TVM_EditMode) && !(Mode & TVM_ReadOnly)) {
         // ====== Tupeldaten initialisieren ================================
         Tuple* tuple = new Tuple;
         CHECK_PTR(tuple);

         tuple->ID         = "";
         tuple->TableName  = Views->BaseTableName;
         tuple->View       = View;
         tuple->Attributes = Attributes.count();

         integer keys = 0;
         for(cardinal i = 0;i < tuple->Attributes;i++) {
            tuple->Attribute[i] = *(Attributes.at(i));
            tuple->IsKey[i]     = false;
            tuple->Value[i]     = item->text(i);

            for(cardinal j = 0;j < ViewTable::MaxPrimaryKeyEntries;j++) {
               if((Views->PrimaryKey[j]) && (*(Attributes.at(i)) == Views->PrimaryKey[j])) {
                  tuple->ID += item->text(i) + "";
                  tuple->IsKey[i] = true;
                  keys++;
                  break;
               }
            }
         }
         for(cardinal i = 0;i < ViewTable::MaxPrimaryKeyEntries;i++) {
            if(Views->PrimaryKey[i] != NULL) {
               keys--;
            }
         }
         if(keys != 0) {
            cerr << "FEHLER: TableViewer::selectTuple(" << Views->BaseTableName
                 << ") - Primrschlssel-Angaben sind fehlerhaft!" << endl;
            exit(1);
         }

         // ====== Ist schon Fenster fr Tupel auf dem Bildschirm? ==========
         TupleEditor* editor = TupleEditors[tuple->ID];
         if(editor != NULL) {
            editor->raise();
            delete tuple;
            return;
         }

         // ====== Suchfunktion aktualisieren ===============================
         SkipNextUpdate   = true;
         MinNotFoundIndex = 0;
         FindLineEdit->setText(item->text(FindSection));

         // ====== Editor ffnen ============================================
         editor = new TupleEditor(Connection,tuple,false,JoinAttribute);
         CHECK_PTR(editor);
         editor->show();
         QObject::connect(editor,SIGNAL(done(TupleEditor*, const bool)),this,SLOT(closeEditor(TupleEditor*, const bool)));
         TupleEditors.insert(tuple->ID,editor);
      }
   }
}


// ###### Qt-Slot: Editor geschlossen #######################################
void TableViewer::closeEditor(TupleEditor* editor, const bool commited)
{
   const Tuple* tuple = editor->getTuple();
   if(commited) {
      loadTable(View->ViewName,View->OrderBy);
      QListViewItem* item = TableView->firstChild();
      while(item != NULL) {
         bool found = true;
         for(cardinal i = 0;i < tuple->Attributes;i++) {
            if(tuple->Value[i] != item->text(i)) {
               found = false;
               break;
            }
         }
         if(found == true) {
            TableView->ensureItemVisible(item);
            TableView->setCurrentItem(item);
            TableView->setFocus();
            raise();
            break;
         }
         item = item->nextSibling();
      }
   }
   TupleEditors.remove(tuple->ID);
}


// ###### Qt-Slot: Attribut angeklickt ######################################
void TableViewer::setSortAttribute(int section)
{
   // ====== Label aktualisieren ============================================
   FindSection = section;
   FindLabel->setText("Auswahl: " + Views->View[0].toTitle(*(Attributes.at(section))));

   // ====== Suche aktualisieren ============================================
   SkipNextUpdate   = true;
   MinNotFoundIndex = 0;
   QListViewItem* item = TableView->firstChild();
   if(item != NULL)
      FindLineEdit->setText(item->text(FindSection));
   else
      FindLineEdit->setText(0);
}



// ###### Qt-Slot: Suchfunktion #############################################
void TableViewer::search(const QString& newText)
{
   // ====== Suche vorbereiten ==============================================
   if(SkipNextUpdate) {
      SkipNextUpdate = false;
      return;
   }
   const char* text          = newText.latin1();
   const cardinal textLength = strlen(text);


   // ====== Tupel in ListView suchen =======================================
   QListViewItem* item = TableView->firstChild();
   while(item != NULL) {
      const QString itemTextValue = item->text(FindSection).stripWhiteSpace();
      const char* itemText        = itemTextValue.latin1();
      if((itemText != NULL) && (!(strncasecmp(itemText,text,textLength)))) {
         TableView->ensureItemVisible(item);
         TableView->setCurrentItem(item);
         MinNotFoundIndex = textLength;
         return;
      }

      item = item->nextSibling();
   }


   // ====== LineEdit aktualisieren =========================================
   MinNotFoundIndex = min(MinNotFoundIndex,textLength);
   FindLineEdit->setText(newText.left(MinNotFoundIndex));
   return;
}


// ###### Leerzeichen vom Stringende entfernen ##############################
QString trim(const QString& string)
{
   const cardinal l = string.length();
   integer i = (integer)l - 1;
   while(i >= 0) {
      if(string[i] != ' ') {
         break;
      }
      i--;
   }
   QString neu(string.mid(0,i + 1));
   return(neu);
}


// ###### Tabelle laden #####################################################
void TableViewer::loadTable(const char* tableName,
                            const char* orderByString)
{
   setCursor(waitCursor);

   // ====== Eingabe vorbereiten ============================================
   QString table(tableName);
   QString orderBy(orderByString);
   if((tableName == NULL) || (orderByString == NULL)) {
      table   = Views->View[0].ViewName;
      orderBy = Views->View[0].OrderBy;
   }


   // ====== Alte Tabelle lschen ===========================================
   TableView->clear();
   const cardinal columns = (cardinal)TableView->columns();
   for(cardinal i = 0;i < columns;i++) {
      TableView->removeColumn(0);
   }
   Attributes.clear();
   FindSection = 0;


   // ====== Transaktion beginnen ===========================================
   try {
      Connection->beginTransaction();


      // ====== Cursor deklarieren ==========================================
      QString command;
      if(JoinAttribute.length() < 1) {
         command = "SELECT * FROM " + table + " ORDER BY " + orderBy;
      }
      else {
         command = "SELECT * FROM " +
                   table + " WHERE " + JoinAttribute + "='" + JoinValue + "'" +
                   " ORDER BY " + orderBy;
      }
      Connection->createCursor("loadTable_Cursor",command);


      // ====== Cursor-Daten kopieren =======================================
      Connection->fetchCursor("loadTable_Cursor","ALL");


      // ====== Attribute aus Datenbank lesen ===============================
      integer selectedTupleColumn = -1;
      FindSection                 = 0;
      QString findAttribute = orderBy;
      int comma = findAttribute.find(',');
      if(comma >= 0) {
         findAttribute = findAttribute.left(comma).stripWhiteSpace();
      }

      const cardinal fields = Connection->getFields();
      for(cardinal i = 0;i < fields;i++) {
         TableView->addColumn(Views->View[0].toTitle(Connection->getField(i)));
         const QString* columnName = new QString(Connection->getField(i));
         Attributes.append(columnName);
         if(*columnName == SelectAttribute) {
            selectedTupleColumn = i;
         }
         if(*columnName == findAttribute) {
            FindSection = i;
         }
      }

      TableView->setSorting(FindSection,true);
      TableView->setAllColumnsShowFocus(true);
      FindLabel->setText("Auswahl: " + Views->View[0].toTitle(*(Attributes.at(FindSection))));


      // ====== Tupel aus Cursor lesen ======================================
      const cardinal tuples = Connection->getTuples();
       for(cardinal i = 0;i < tuples;i++) {
         QListViewItem* item = new QListViewItem(TableView);
         CHECK_PTR(item);
         for(cardinal j = 0;j < fields;j++) {
            QString value = Connection->getValue(i,j);
            value = trim(value);
            item->setText(j,value);
            if(((int)j == selectedTupleColumn) && (SelectValue == value)) {
               TableView->setSelected(item,true);
            }
         }
      }


      // ====== Transaktion beenden =========================================
      Connection->deleteCursor("loadTable_Cursor");
      Connection->commitTransaction();
      QString countString;
      countString.setNum(tuples);
      StatusBar->setText("Es wurden " + countString + " Tupel aus der Datenbank geladen.");
   }

   // ====== Fehler abfangen ================================================
   catch(SQLException e) {
      StatusBar->setText("Fehler beim Einlesen der Tabelle!");
      qApp->processEvents();
      Connection->rollbackTransaction();
      sqlWarning(this,e.toString());
   }
   setCursor(arrowCursor);
}
