// ##########################################################################
// ####                                                                  ####
// ####                         Prfungsamt-Client                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Tupel-Editor                                                     ####
// ####                                                                  ####
// #### Version 1.00  --  12. Juli 2000                                  ####
// ####                                                                  ####
// #### Copyright (C) 2000  Thomas Dreibholz                             ####
// ####                     Universitt Bonn                             ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "tupleeditor.h"
#include "sqlmessages.h"


#include <qlayout.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qhbuttongroup.h>


#include "tupleeditor.moc"


// ###### Constructor #######################################################
TupleEditor::TupleEditor(SQLConnectionInterface* connection,
                         Tuple*                  tuple,
                         const bool              newTuple,
                         const QString&          joinAttribute,
                         QWidget*                parent,
                         const char*             name)
   : QWidget(parent,name)
{
   Connection  = connection;
   EditorTuple = tuple;
   NewTuple    = newTuple;
   Selector    = NULL;


   // ====== Widget initialisieren ==========================================
   WhatsThis = new QWhatsThis(this);
   CHECK_PTR(WhatsThis);
   WhatsThis->add(this,"Dies ist der Tupel-Editor.");
   QGridLayout* topLayout = new QGridLayout(this,4,2,5);
   CHECK_PTR(topLayout);

   topLayout->setColStretch(0,5);
   topLayout->setColStretch(1,0);
   topLayout->setRowStretch(0,0);
   topLayout->setRowStretch(1,0);
   topLayout->setRowStretch(2,10);
   topLayout->setRowStretch(3,0);


   QLabel* label = new QLabel(EditorTuple->View->toTitle(EditorTuple->TableName),this);
   CHECK_PTR(label);
   WhatsThis->add(label,EditorTuple->View->toHelp(EditorTuple->TableName));
   QFont font = label->font();
   font.setPointSize(20);
   label->setFont(font);
   label->setAlignment(Qt::AlignCenter);
   label->setMinimumSize(label->sizeHint());
   topLayout->addMultiCellWidget(label,0,0,0,1);


   // ====== Editor-Widgets =================================================
   QWidget* editorGroup = new QWidget(this);
   CHECK_PTR(editorGroup);
   WhatsThis->add(editorGroup,"Hier kann das Tupel editiert werden.");
   QGridLayout* editorLayout = new QGridLayout(editorGroup,EditorTuple->Attributes,2,5);
   CHECK_PTR(editorLayout);
   editorLayout->setColStretch(0,0);
   editorLayout->setColStretch(1,10);

   AttrEdit = new AttributeEditor*[EditorTuple->Attributes];
   CHECK_PTR(AttrEdit);
   for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
      // ====== Label mit Attributnamen =====================================
      QLabel* label = new QLabel(EditorTuple->View->toTitle(EditorTuple->Attribute[i]),editorGroup);
      CHECK_PTR(label);
      WhatsThis->add(label,"Dies ist der Name eines Attributes: " +
                           QString(EditorTuple->View->toHelp(EditorTuple->Attribute[i])) +
                           ".");
      if(EditorTuple->IsKey[i] == true) {
         QFont font = label->font();
         font.setBold(true);
         label->setFont(font);
      }
      label->setMinimumSize(label->sizeHint());
      editorLayout->addWidget(label,i,0);

      // ====== AttributeEditor fr den Wert ================================
      AttrEdit[i] = new AttributeEditor(Connection,
                                        EditorTuple->View,EditorTuple->Attribute[i],
                                        (EditorTuple->Attribute[i] != joinAttribute),
                                        EditorTuple->Value[i],editorGroup);
      CHECK_PTR(AttrEdit[i]);
      editorLayout->addWidget(AttrEdit[i],i,1);
      WhatsThis->add(AttrEdit[i],"Dies ist der Wert des Attributes:" +
                                 EditorTuple->View->toTitle(EditorTuple->Attribute[i]) +
                                  ".");
      QObject::connect(AttrEdit[i],SIGNAL(userUpdate()),this,SLOT(update()));
   }
   topLayout->addWidget(editorGroup,1,0);


   // ====== Verknpfungen ==================================================
   if(EditorTuple->View->Joins > 0) {
      QTabWidget* tabGroup = new QTabWidget(this);
      CHECK_PTR(tabGroup);
      WhatsThis->add(tabGroup,"Hier knnen zum Tupel gehrende Verbindungen "
                              "editiert werden.");
      QIconSet icon1(QPixmap("folder.xpm"));

      for(cardinal j = 0;j < EditorTuple->View->Joins;j++) {
         QString joinValue;
         const JoinEntry* join = &EditorTuple->View->Join[j];
         if(join != NULL) {
            for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
               if(!(strcmp(EditorTuple->Attribute[i],join->JoinAttributeR1))) {
                  joinValue = EditorTuple->Value[i];
                  break;
               }
            }
         }
         TableViewer* viewer = new TableViewer(Connection,
                                      join->View,
                                      TableViewer::TVM_EditMode,
                                      join->JoinAttributeR2,
                                      joinValue,
                                      NULL,NULL,
                                      tabGroup);
         CHECK_PTR(join);
         viewer->loadTable();
         tabGroup->addTab(viewer,
                          QIconSet(QPixmap(join->View->PixmapName)),
                          EditorTuple->View->Join[j].JoinTitle);
      }
      topLayout->addMultiCellWidget(tabGroup,1,4,1,1);

      if(NewTuple) {
         tabGroup->setEnabled(false);
      }
   }


   // ====== Kontroll-Buttons ===============================================
   QHButtonGroup* controlGroup = new QHButtonGroup("Datenbank-Zugriff",this);
   CHECK_PTR(controlGroup);
   topLayout->addWidget(controlGroup,3,0);
   WhatsThis->add(controlGroup,"Diese Gruppe enthlt Funktionen fr den "
                               "Datenbank-Zugriff: Speichern, Lschen und Abbruch.");

   QPushButton* updateButton = new QPushButton((NewTuple == true) ?
                                               "Einfgen" : "Aktualisieren",
                                               controlGroup);
   CHECK_PTR(updateButton);
   QPushButton* deleteButton = new QPushButton("Lschen",controlGroup);
   CHECK_PTR(deleteButton);
   if(NewTuple) {
      deleteButton->setEnabled(false);
   }
   QPushButton* abortButton = new QPushButton("Zurck",controlGroup);
   CHECK_PTR(abortButton);

   WhatsThis->add(updateButton,"Hiermit werden die nderungen am Tupel gespeichert.");
   WhatsThis->add(abortButton,"Hiermit werden die nderungen zurckgenommen.");
   WhatsThis->add(deleteButton,"Hiermit wird das Tupel gelscht.");

   updateButton->setMinimumSize(updateButton->sizeHint());
   deleteButton->setMinimumSize(deleteButton->sizeHint());
   abortButton->setMinimumSize(abortButton->sizeHint());


   // ====== Signale mit Slots verbinden ====================================
   QObject::connect(updateButton,SIGNAL(clicked()),this,SLOT(update()));
   QObject::connect(abortButton,SIGNAL(clicked()),this,SLOT(abort()));
   QObject::connect(deleteButton,SIGNAL(clicked()),this,SLOT(remove()));

   setCaption("Tupel-Editor");
   if(AttrEdit[0] != NULL) {
      AttrEdit[0]->setFocus();
   }
}


// ###### Destructor ########################################################
TupleEditor::~TupleEditor()
{
   delete WhatsThis;
   WhatsThis = NULL;
   delete AttrEdit;
   AttrEdit = NULL;
   delete EditorTuple;
   EditorTuple = NULL;
   if(Selector) {
      delete Selector;
      Selector = NULL;
   }
}


// ###### Qt-Slot: Fenster geschlossen ######################################
void TupleEditor::closeEvent(QCloseEvent* event)
{
   event->accept();
   emit done(this,false);
}


// ###### Tupel-Struktur zurckgeben ########################################
const Tuple* TupleEditor::getTuple() const
{
   return(EditorTuple);
}


// ###### SQL-Update bzw. Insert durchfhren ################################
void TupleEditor::update()
{
   // ====== Transaktion beginnen ===========================================
   try {
      Connection->beginTransaction();


      // ====== Tupel einfgen ==============================================
      bool changed = false;
      if(NewTuple) {
         changed = true;

         // ====== Befehlszeile zusammenstellen =============================
         QString command = "INSERT INTO " + EditorTuple->TableName + "(";
         bool firstUsage = true;
         for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
            if(AttrEdit[i]->getValue().length() > 0) {
               if(!firstUsage) {
                  command += ",";
               }
               firstUsage = false;
               command += EditorTuple->Attribute[i];
            }
         }
         command += ") VALUES (";
         firstUsage = true;
         for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
            if(AttrEdit[i]->getValue().length() > 0) {
               if(!firstUsage) {
                  command += ",";
               }
               firstUsage = false;
               command += "'" + AttrEdit[i]->getValue() + "'";
            }
         }
         command += ")";

         // ====== Befehl ausfhren =========================================
         if(!firstUsage) {
            Connection->execute(command);
            for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
               EditorTuple->Value[i] = AttrEdit[i]->getValue();
            }
         }
         else {
            sqlWarning(this,"Es wurde nichts eingegeben!");
            Connection->rollbackTransaction();
            return;
         }
      }


      // ====== Updates durchfhren =========================================
      else {
         // ====== Schlssel-Werte sichern ==================================
         QString value[EditorTuple->Attributes];
         for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
            value[i] = EditorTuple->Value[i];
         }


         // ====== Update der Schlssel-Attribute ===========================
         QString command = "UPDATE " + EditorTuple->TableName + " ";
         for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
            if((AttrEdit[i]->getValue() != EditorTuple->Value[i]) /* &&
               (EditorTuple->IsKey[i]) */) {
               if(changed == false) {
                  changed = true;
                  command += "SET " + EditorTuple->Attribute[i];
               }
               else {
                  command += ", " + EditorTuple->Attribute[i];
               }

               // ====== Befehlszeile zusammenstellen =======================
               if(AttrEdit[i]->getValue().length() > 0) {
                  command += "='" + AttrEdit[i]->getValue() + "'";
               }
               else {
                  command += "=NULL";
               }
            }
         }
         // ====== Befehl ausfhren =========================================
         if(changed == true) {
            command += " WHERE ";
            bool firstKey = true;
            for(cardinal j = 0;j < EditorTuple->Attributes;j++) {
               if(EditorTuple->IsKey[j]) {
                  if(firstKey) {
                     firstKey = false;
                  }
                  else {
                     command += " AND ";
                  }
                  command += EditorTuple->Attribute[j] + "='" + value[j] + "'";
                  value[j] = AttrEdit[j]->getValue();
               }
            }
            Connection->execute(command);
         }
      }


      // ====== Transaktion beenden =========================================
      Connection->commitTransaction();
      for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
         EditorTuple->Value[i] = AttrEdit[i]->getValue();
      }
      emit done(this,changed);
   }

   // ====== Fehler abfangen ================================================
   catch(SQLException e) {
      Connection->rollbackTransaction();
      sqlWarning(this,e.toString());
   }
}


// ###### SQL-Delete durchfhren ############################################
void TupleEditor::remove()
{
   if(NewTuple) {
      return;
   }

   // ====== Transaktion beginnen ===========================================
   try {
      Connection->beginTransaction();

      // ====== Befehlszeile zusammenstellen ================================
      QString command = "DELETE FROM " + EditorTuple->TableName + " WHERE ";
      bool firstKey = true;
      for(cardinal i = 0;i < EditorTuple->Attributes;i++) {
         if(EditorTuple->IsKey[i]) {
            if(firstKey) {
               firstKey = false;
            }
            else {
               command += " AND ";
            }
            command += EditorTuple->Attribute[i] + "='" + EditorTuple->Value[i] + "'";
         }
      }

      // ====== Befehl ausfhren ============================================
      Connection->execute(command);

      // ====== Transaktion beenden =========================================
      Connection->commitTransaction();
      emit done(this,true);
   }

   // ====== Fehler abfangen ================================================
   catch(SQLException e) {
      Connection->rollbackTransaction();
      sqlWarning(this,e.toString());
   }
}


// ###### Qt-Slot: Abbruch ##################################################
void TupleEditor::abort()
{
   emit done(this,false);
}
